! ===========================================================================
  program test_super
! ===========================================================================

  implicit none
  integer, parameter              :: neqns = 23, maxlnz = 173, maxsub = 62
  integer                         :: nnode, i
  integer, dimension(neqns+1)     :: ixvec1, ixvec2, inode
  integer, dimension(maxsub)      :: ixsub
  real(8), dimension(neqns)       :: dia
  real(8), dimension(maxlnz)      :: xspars
  
  real(8), dimension(neqns,neqns) :: hh, hh1, hh2, hh3

! example matrix
  call set_example; call expand( hh, .true. )

! determine supernode partitioning
  call super_nodes( neqns, ixvec1, ixvec2, ixsub, nnode, inode )

  do i = 1, nnode
     write(*,'(1x,3(a,i3))') 'node',i,' from row', inode(i+1)+1,'  to row', &
&                                                                 inode(i)
  end do
  write(*,*)'Matrix'; call print_matrix( hh )

! Cholesky factorisation
  call super_gsfct( neqns, ixvec1, xspars, ixvec2, ixsub, dia, nnode, inode )
  call expand( hh1, .false. )
  hh2 = matmul( hh1, transpose(hh1) )
  write(*,*)'Cholesky factor'; call print_matrix( hh1 )

! Matrix inverse
  call super_sparsinv( neqns, ixvec1, xspars, ixvec2, ixsub, dia, nnode, inode )
  call expand( hh1, .true. ); hh1 = hh1 * 10000
  write(*,*)'"Inverse" x 10000'; call print_matrix( hh1 )

  contains

! =======================
  subroutine set_example
! =======================

  ixvec1 = (/ 1, 11, 20, 30, 39, 49, 58, 68, 77, 87, 96, 108, 119, 129, 138, &
&             146, 153, 159, 164, 168, 171, 173, 174, 174 /)
  ixvec2 = (/  1, 2, 11, 12, 21, 22, 31, 32, 41, 42, 51, 52, 53, 54, 55, 56, &
&              57, 58, 59, 60, 61, 62, 62, 62 /)

  ixsub = (/ 2, 11, 12, 13, 14, 15, 16, 17, 18, 23,  4, 11, 12, 13, 14, 15,  &
&           16, 17, 18, 23,  6, 11, 12, 13, 14, 15, 16, 17, 18, 23,  8, 11,  &
&           12, 13, 14, 15, 16, 17, 18, 23, 10, 15, 16, 17, 18, 19, 20, 21,  &
&           22, 23, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23 /)

  dia = (/ 510.0, 520.0, 530.0, 540.0, 550.0, 560.0, 570.0, 580.0, 590.0, &
&          600.0, 610.0, 620.0, 630.0, 640.0, 650.0, 660.0, 670.0, 680.0, &
&          690.0, 700.0, 710.0, 720.0, 730.0 /)

  xspars = (/ 7.0, 16.0, 17.0, 18.0, 19.0, 20.0, 21.0, 22.0, 23.0, 28.0, 21.0, &
&            22.0, 23.0, 24.0, 25.0, 26.0, 27.0, 28.0, 33.0, 19.0, 26.0, 27.0, &
&            28.0, 29.0, 30.0, 31.0, 32.0, 33.0, 38.0, 31.0, 32.0, 33.0, 34.0, &
&            35.0, 36.0, 37.0, 38.0, 43.0, 31.0, 36.0, 37.0, 38.0, 39.0, 40.0, &
&            41.0, 42.0, 43.0, 48.0, 41.0, 42.0, 43.0, 44.0, 45.0, 46.0, 47.0, &
&            48.0, 53.0, 43.0, 46.0, 47.0, 48.0, 49.0, 50.0, 51.0, 52.0, 53.0, &
&            58.0, 51.0, 52.0, 53.0, 54.0, 55.0, 56.0, 57.0, 58.0, 63.0, 55.0, &
&            60.0, 61.0, 62.0, 63.0, 64.0, 65.0, 66.0, 67.0, 68.0, 65.0, 66.0, &
&            67.0, 68.0, 69.0, 70.0, 71.0, 72.0, 73.0, 67.0, 68.0, 69.0,  0.0, &
&             0.0,  0.0,  0.0, 74.0, 75.0, 76.0, 77.0,  0.0, 73.0, 74.0,  0.0, &
&             0.0,  0.0,  0.0, 79.0, 80.0, 81.0, 82.0,  0.0, 79.0,  0.0,  0.0, &
&             0.0,  0.0, 84.0, 85.0, 86.0, 87.0,  0.0,  0.0,  0.0,  0.0,  0.0, &
&            89.0, 90.0, 91.0, 92.0,  0.0, 91.0, 92.0, 93.0, 94.0, 95.0, 96.0, &
&            97.0, 98.0, 97.0, 98.0, 99.0, 100.0, 101.0, 102.0, 103.0, 103.0,  &
&           104.0, 105.0, 106.0, 107.0, 108.0, 109.0, 110.0, 111.0, 112.0,     &
&           113.0, 115.0,  0.0,  0.0, 118.0,  0.0,  0.0, 123.0, 127.0, 128.0,  &
&           133.0 /)

  return
  end subroutine set_example

! =============================
  subroutine expand( hhh, opt )
! =============================

  implicit none
  real(8), dimension(neqns,neqns), intent(out) :: hhh
  logical, intent(in)                          :: opt
  integer                                      :: irow, ksub, i, icol

! expand sparse-stored matrix into two-dimensional array
  hhh = 0.d0
  do irow = 1, neqns
     hhh(irow,irow) = dia(irow); ksub = ixvec2(irow)
     do i = ixvec1(irow), ixvec1(irow+1)-1
        icol = ixsub(ksub); ksub = ksub + 1
        hhh(icol,irow) = xspars(i); if( opt ) hhh(irow,icol) = xspars(i)
     end do
  end do

  return
  end subroutine expand

! =============================
  subroutine print_matrix( hh )
! =============================

  implicit none
  real(8), dimension(neqns,neqns), intent(in) :: hh
  integer                                     :: i

  do i = 1, neqns
     write(*,'(i2,(t4,12f10.4))') i, hh(i,:i)
  end do

  return
  end subroutine print_matrix

  end program test_super
